#!/usr/bin/env python
#
# pyargocat.py version 0.1
#
# Little Python Script to manage ARGO Navis DSC catalogs via serial port. It can manage the three
# types of internal catalogs (user, comet, asteroid) by feeding the device with a properly formatted
# text file (read the ARGO Navis DSC manual for the format or check some examples)
#
# Requires Python >= 2.7 due to usage of argparse and pyserial module.
#
# 
# 2012-2013 Federico Pellegrin <fede@evolware.org>
#
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#

import serial
import argparse
import time

def sendArgoCmd(ser,sendStr,checkEcho,hasResp):
    ser.flushInput()
    ser.write(sendStr+"\n")
    ser.flush()
    rline=ser.readline()

    if checkEcho:
        if not(sendStr in rline):
            return (-1,"")
    if hasResp:
        rline=ser.readline()

    return (0,rline)

if __name__=='__main__':
    argpar=argparse.ArgumentParser(description='Command-line Argo Navis catalog manager (version 0.1).')
    argpar.add_argument("-b", "--baud", help="Serial Baud Rate", type=int, choices=[9600,19200,38400,57600], default=38400);
    argpar.add_argument("-d", "--dev", help="Serial Device", default="/dev/ttyUSB0");
    argpar.add_argument("-c", "--catalog", help="Catalog to Work", type=str, choices=['user','comet','asteroid'], default='user');
    argpar.add_argument("-f", "--file", help="File to Import to Catalog", type=file);    
    argpar.add_argument("-p", "--purge", help="Purge Catalog (prior to import)", action="store_true", default=False);    
    argpar.add_argument("-v", "--version", action='version', version='%(prog)s 0.1')
    args = argpar.parse_args()


    # print args.baud    
    ser=serial.Serial(args.dev,args.baud,timeout=3)


    filelines=()
    if args.file:
        filelines=args.file.readlines()
        print "Input file contains "+str(len(filelines))+" entries to import"


    print "Remember to set Argo Navis in \"SETUP LOAD CAT\" mode from the Setup Menu Now!"
    time.sleep(3)

    (rval,rstr)=sendArgoCmd(ser,"navis",True,True)
    if "%" not in rstr:
        (rval,rstr)=sendArgoCmd(ser,"#Quit",True,True)


    (rval,rstr)=sendArgoCmd(ser,args.catalog,True,True)
    if rval!=0 or ('OK' not in rstr):
        rval=-1

    if rval==0:
        (rval,rstr)=sendArgoCmd(ser,"#Memory",True,True)
        if rval==0:
            print "Memory space in bytes: "+rstr.rstrip("\n")

        (rval,rstr)=sendArgoCmd(ser,"#Count",True,True)
        if rval==0:
            print "Elements in catalog: "+rstr.rstrip("\n")

        if args.purge:
            print "Purging current catalog... "
            (rval,rstr)=sendArgoCmd(ser,"#Purge",True,True)
            if rval==0 and ('OK' in rstr):
                print "Purge OK!"
            else:
                print "Purge ERROR!"

            (rval,rstr)=sendArgoCmd(ser,"#Memory",True,True)
            if rval==0:
                print "Memory space after purge in bytes: "+rstr.rstrip("\n")

        
        if len(filelines) > 0:
            print "Inserting data now, this may take some time..."
            cntr=0
            for cline in filelines:
                (rval,rstr)=sendArgoCmd(ser,cline.rstrip("\n").rstrip("\r"),True,True)
                if rval!=0 or ('OK' not in rstr):
                    if 'exceeded' in rstr:
                        print "Error: Maximum number of objects reached for the catalog!"
                        break;
                    else:
                        print "Error inserting: "+rstr.rstrip("\n").rstrip("\r")+" (line="+str(cntr)+")"
                    # Error string sometimes even has an OK after :-? Try and discard in case
                    rstr=ser.readline()
                    print rstr
                cntr+=1


        if len(filelines) > 0 or args.purge:
            # Write to flash often takes more time
            ser.timeout=20
            (rval,rstr)=sendArgoCmd(ser,"#Write",True,True)
            print "Writing to Flash: "+rstr.rstrip("\n")
            ser.timeout=3


        (rval,rstr)=sendArgoCmd(ser,"#Quit",True,False)
    else:
        print "Problems entering in catalog mode: is the Argo Navis connected and in LOAD CAT mode?"

    ser.close()
